<?php
namespace Teknisa\CUS\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;

class Cus02000AcompanhamentoDePrecoUnitario extends \Zeedhi\Framework\Controller\Report {
  protected $novoCodigo;
  protected $consplan;
  protected $birtStrategy;
  protected $environment;
  protected $connection;
  protected $entityManager;
  protected $AcompanhamentoDePrecoUnitarioService;

  public function __construct($reportService, $novoCodigo, $consplan, $birtStrategy, $environment, $util, StoredProcedure $procedureConsplanGeral, $connection, $entityManager, $Cus02000AcompanhamentoDePrecoUnitarioService) {
    $this->novoCodigo                             = $novoCodigo;
    $this->consplan                               = $consplan;
    $this->reportService                          = $reportService;
    $this->birtStrategy                           = $birtStrategy;
    $this->environment                            = $environment;
    $this->util                                   = $util;
    $this->procedureConsplanGeral                 = $procedureConsplanGeral;
    $this->connection                             = $connection;
    $this->entityManager                          = $entityManager;
    $this->AcompanhamentoDePrecoUnitarioService  = $Cus02000AcompanhamentoDePrecoUnitarioService;
  }
  
  protected $reportMapping = array (
    'CUS02000' => 
    array (
      'strategy' => 'BIRT',
      'parameterMapping' => 
      array (
        "CDFILIAL"      => "P_CDFILIAL",
        "LISTASERV"     => "P_CDSERVFILI",
        "NRCONSPLAN"    => "P_NRCONSPLAN",
        "DTINICIAL"     => "P_DTINIC",
        "DTFINAL"       => "P_DTFINA",
        "CDTIPOCUSTO"   => "P_NRTIPOCUSTO",
        "NRORG"         => "P_NRORG"
      )
    ),
    'CUS02001' => 
    array (
      'strategy' => 'BIRT',
      'parameterMapping' => 
      array (
        "CDFILIAL"      => "P_CDFILIAL",
        "LISTASERV"     => "P_CDSERVFILI",
        "NRCONSPLAN"    => "P_NRCONSPLAN",
        "DTINICIAL"     => "P_DTINIC",
        "DTFINAL"       => "P_DTFINA",
        "CDTIPOCUSTO"   => "P_NRTIPOCUSTO",
        "NMTIPOCUSTO"   => "P_NMTIPOCUSTO",
        "NRORG"         => "P_NRORG"
      )
    )    
  );
  
  public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
    $row = $request->getRow();
    
    $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
    
    $row['NRORG'] = $this->environment->getNrorg();
    $row["NMGRUPSERV"] = !$row["NMGRUPSERV"] ? "T" : implode("_", $row["NMGRUPSERV"]);
    $row["CDOPERADOR"] = $this->environment->getCdOperador();
    $nrConsPlan = $this->novoCodigo->getCodeByTableName('CONSPLAN'.$row['CDFILIAL'], $row['NRORG']);
    
    $dataInicial = \DateTime::createFromFormat('d/m/Y', $row['DTINICIAL']);
    $dataFinal   = \DateTime::createFromFormat('d/m/Y', $row['DTFINAL']);
    
    if(!$row["NMSERVICO"]) {
      $params = [
          "CDFILIAL"      => $row["CDFILIAL"],
          "CDGRUPFILI"    => "T",
          "CDGRUPSERV"    => $row["NMGRUPSERV"],
          "CDOPERADOR"    => $row["CDOPERADOR"]
      ];      
      
      $servicos = $this->AcompanhamentoDePrecoUnitarioService->getAllServices($params);
      $cdServFili = array_map(function($servico){
        return $servico["CDSERVFILI"];
      }, $servicos);
      $row['LISTASERV'] = implode("_", $cdServFili);
    } else {
      $row['LISTASERV'] = implode("_", $row["NMSERVICO"]);
    }
    
    $row['LISTASERV'] = $row['LISTASERV'] == "" ? "X" : $row['LISTASERV'];
    
    $connection = $this->entityManager->getConnection();
    $procedureConsumoPlanejado = new StoredProcedure($connection, 'CONSUMO_PLANEJADO_PLA');
    $procedureConsumoPlanejado->addParam(new Param('P_FILIAL',       Param::PARAM_INPUT, $row['CDFILIAL'],                    Param::PARAM_TYPE_STR));
    $procedureConsumoPlanejado->addParam(new Param('P_SERVICOS',     Param::PARAM_INPUT, $row['LISTASERV'],                   Param::PARAM_TYPE_STR));
    $procedureConsumoPlanejado->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $row['DTINICIAL'],                   Param::PARAM_TYPE_STR));
    $procedureConsumoPlanejado->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT, $row['DTFINAL'],                     Param::PARAM_TYPE_STR));
    $procedureConsumoPlanejado->addParam(new Param('P_IDARQUIVO',    Param::PARAM_INPUT, "T",                                 Param::PARAM_TYPE_STR));
    $procedureConsumoPlanejado->addParam(new Param('P_IDPRODUTOS',   Param::PARAM_INPUT, "CFP",                               Param::PARAM_TYPE_STR));
    $procedureConsumoPlanejado->addParam(new Param('P_IDSEPPRATO',   Param::PARAM_INPUT, "P",                                 Param::PARAM_TYPE_STR));
    $procedureConsumoPlanejado->addParam(new Param('P_PEACRESCIMO',  Param::PARAM_INPUT, 0,                                   Param::PARAM_TYPE_INT));
    $procedureConsumoPlanejado->addParam(new Param('P_IDARREDOND',   Param::PARAM_INPUT, "N",                                 Param::PARAM_TYPE_STR));
    $procedureConsumoPlanejado->addParam(new Param('P_NRCONSPLAN',   Param::PARAM_INPUT, $nrConsPlan,                         Param::PARAM_TYPE_STR));
    $procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, '0',                                 Param::PARAM_TYPE_STR));
    $procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, 'zzzzzzzzzz',                        Param::PARAM_TYPE_STR));
    $procedureConsumoPlanejado->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $row['CDOPERADOR'],                  Param::PARAM_TYPE_STR));
    $procedureConsumoPlanejado->addParam(new Param('P_STDTINICIAL',  Param::PARAM_INPUT, "_".$dataInicial->format('Ymd')."_", Param::PARAM_TYPE_STR));
    $procedureConsumoPlanejado->addParam(new Param('P_STDTFINAL',    Param::PARAM_INPUT, "_".$dataFinal->format('Ymd')."_",   Param::PARAM_TYPE_STR));
    $procedureConsumoPlanejado->execute();
    
    $row["NRCONSPLAN"] = $nrConsPlan;
  }
  
}